rem TBD: current not compiling - need to check ...

cmr "install llvm prebuilt" --version=16.0.4
cmr "install llvm prebuilt" --version=17.0.6

cmr "get lib onnxruntime lang-cpp _cpu" --version=1.11.1
cmr "get lib onnxruntime lang-cpp _cpu" --version=1.13.1
cmr "get lib onnxruntime lang-cpp _cpu" --version=1.15.1
