#!/bin/bash

CUR_DIR=${PWD}

echo ""
echo "Current execution path: ${CUR_DIR}"
echo "Path to script: ${CM_TMP_CURRENT_SCRIPT_PATH}"



container="docker"


if [ "${container}" = "docker" ]; then

  echo "==================  Install Docker container (you can skip if already installed)=================="

  sudo apt-get update
  sudo apt-get -y install \
      apt-transport-https \
      ca-certificates \
      curl \
      gnupg \
      lsb-release \
      tar 

  # Add Docker’s official GPG key
  curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo gpg --dearmor -o /usr/share/keyrings/docker-archive-keyring.gpg

  echo \
    "deb [arch=$(dpkg --print-architecture) signed-by=/usr/share/keyrings/docker-archive-keyring.gpg] https://download.docker.com/linux/ubuntu \
    $(lsb_release -cs) stable" | sudo tee /etc/apt/sources.list.d/docker.list > /dev/null

  sudo apt-get update
  sudo apt-get -y install docker-ce docker-ce-cli containerd.io

  sudo usermod -aG docker $USER

  su - $USER

else 

echo "==================  Install Podman container (you can skip if already installed)=================="

sudo apt-get update 
sudo apt-get -y install podman
su - $USER

fi 
