echo.

del /Q /S %CM_ARIA2_DOWNLOAD_FILE%

wget --no-check-certificate %CM_ARIA2_DOWNLOAD_URL%
IF %ERRORLEVEL% NEQ 0 EXIT %ERRORLEVEL%

unzip -o -q %CM_ARIA2_DOWNLOAD_FILE%
IF %ERRORLEVEL% NEQ 0 EXIT %ERRORLEVEL%
