#!/bin/bash

echo ""
echo "    bash --init-file ${CM_VIRTUAL_ENV_SCRIPTS_PATH}/activate"
echo ""
echo "    Enter exit to exit virtual env."
echo ""

bash --init-file ${CM_VIRTUAL_ENV_SCRIPTS_PATH}/activate
