del /Q %CM_JAVA_PREBUILT_FILENAME%.zip

wget --no-check-certificate %CM_JAVA_PREBUILT_URL%%CM_JAVA_PREBUILT_FILENAME%.zip
IF %ERRORLEVEL% NEQ 0 EXIT %ERRORLEVEL%

unzip %CM_JAVA_PREBUILT_FILENAME%.zip
IF %ERRORLEVEL% NEQ 0 EXIT %ERRORLEVEL%

del /Q %CM_JAVA_PREBUILT_FILENAME%.zip
