export PATH=${CM_CONDA_BIN_PATH}:$PATH

export CALIBRATION_ANNOTATIONS=${CM_DATASET_CALIBRATION_ANNOTATIONS_FILE_PATH}

export CALIBRATION_DATA_DIR=${CM_CALIBRATION_DATASET_PATH}
export MODEL_CHECKPOINT=${CM_ML_MODEL_FILE_WITH_PATH}

cd ${CM_HARNESS_CODE_ROOT}

bash run_calibration.sh
test "$?" -eq 0 || exit "$?"
