IF NOT DEFINED CM_TMP_CURRENT_SCRIPT_PATH SET CM_TMP_CURRENT_SCRIPT_PATH=%CD%

rem %CM_PYTHON_BIN_WITH_PATH% --version

%CM_PYTHON_BIN_WITH_PATH% %CM_TMP_CURRENT_SCRIPT_PATH%\app.py
IF %ERRORLEVEL% NEQ 0 EXIT %ERRORLEVEL%

echo CM_NEW_VAR_FROM_RUN=XYZ > tmp-run-env.out
