#! /bin/bash

#export CM_CACHE="--no-cache"

export CM_DOCKER_ORG=modularcm
export CM_DOCKER_NAME="mlperf-inference"
export CM_OS_NAME="ubuntu"
export CM_HW_TARGET="cpu"
export CM_OS_VERSION="22.04"

