echo "%CM_JAVA_BIN_WITH_PATH%"
echo.

"%CM_JAVAC_BIN_WITH_PATH%" %CM_TMP_CURRENT_SCRIPT_PATH%\code.java
IF %ERRORLEVEL% NEQ 0 EXIT %ERRORLEVEL%

"%CM_JAVA_BIN_WITH_PATH%" -classpath "%CM_TMP_CURRENT_SCRIPT_PATH%" code
IF %ERRORLEVEL% NEQ 0 EXIT %ERRORLEVEL%
