rem connect CM portable scripts with CK env

set CM_ML_TORCH_MODEL_NAME=resnet50
set CM_ML_MODEL_INPUT_DATA_TYPE=float32
set CM_ML_MODEL_IMAGE_HEIGHT=224
set CM_ML_MODEL_IMAGE_WIDTH=224

rem set CM_DATASET_IMAGENET_PREPROCESSED_DIR=%CM_DATASET_PREPROCESSED_PATH%

set CM_DATASET_IMAGENET_PREPROCESSED_DIR=%CM_DATASET_PREPROCESSED_FULL_PATH%
set CM_CAFFE_IMAGENET_SYNSET_WORDS_TXT=%CM_DATASET_AUX_PATH%\synset_words.txt
set CM_DATASET_IMAGENET_PREPROCESSED_DATA_TYPE=float32
set CM_RESULTS_DIR=%CM_TMP_CURRENT_SCRIPT_PATH%\results
set ML_MODEL_DATA_LAYOUT=NCHW

%CM_PYTHON_BIN_WITH_PATH% -m pip install -r %CM_TMP_CURRENT_SCRIPT_PATH%\requirements.txt
IF %ERRORLEVEL% NEQ 0 EXIT %ERRORLEVEL%

%CM_PYTHON_BIN_WITH_PATH% %CM_TMP_CURRENT_SCRIPT_PATH%\src\pytorch_classify_preprocessed.py
IF %ERRORLEVEL% NEQ 0 EXIT %ERRORLEVEL%
